package pjplugin.builder;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

//import nz.ac.auckland.PYJAMA.//PYJAMALog;
//import nz.ac.auckland.PYJAMA.Activator;
//import nz.ac.auckland.PYJAMA.builder.MarkerInfo;
//import nz.ac.auckland.PYJAMA.builder.MarkerManager;
//import nz.ac.auckland.PYJAMA.builder.PYJAMAFileBuilder.StringOutputStream;
//import nz.ac.auckland.PYJAMA.internal.builder.BuildDeltaVisitor;
//import nz.ac.auckland.PYJAMA.internal.builder.BuildVisitor;
//import nz.ac.auckland.PYJAMA.internal.builder.StreamGobbler;
import pjplugin.Activator;
import pjplugin.constants.PJConstants;
import pjplugin.preferences.PreferenceConstants;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Two way indirection is tricky. We need to get the error lines from
 * Java, to PTJava, and then to Pyjama.
 * 
 * @author vikassingh
 *
 */
/**
 * @author sriram
 * 
 */
public class FileBuilder extends IncrementalProjectBuilder {

	private static final String PYJAMA_FILE_EXTENSION = ".pj";

	/**
	 * If not running in debug mode these files are deleted
	 */
	private static final String TODELETE_FILE_EXTENSION = ".ptjava";

	/**
	 * where the PYJAMA compiler jar file is
	 */
	private static String fCompilerPath;

	/**
	 * whether PYJAMA files should be built or not
	 */
	private static boolean fCanBuild;

	/**
	 * whether PYJAMA files should be compiled using a custom compiler or the
	 * default one
	 */
	private static boolean fUseCustomCompiler;

	/**
	 * specifies the type of project created
	 */
	public static ProjectType projectType = ProjectType.DESKTOP;

	public static String JAVA_CMD_OPTION = "java -jar ";

	public static String ANDROID_CMD_OPTION = "A";

	/**
	 * Listener for when options are changed in project preferences
	 */
	private final IPropertyChangeListener fPropertyChangeListener = new IPropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent event) {
			// check if event change is for the compiler path
			if (event.getProperty().equals(
					PreferenceConstants.PYJAMA_COMPILER_PATH)) {
				// change compiler path
				IPreferenceStore prefs = Activator.getDefault()
						.getPreferenceStore();
				fCompilerPath = prefs
						.getString(PreferenceConstants.PYJAMA_COMPILER_PATH);
				// System.out.println("file");
			}
			if (event.getProperty().equals(
					PreferenceConstants.PYJAMA_ASSOCIATE_NATURE)) {
				// add or remove builder from project
				IPreferenceStore prefs = Activator.getDefault()
						.getPreferenceStore();
				fCanBuild = prefs
						.getBoolean(PreferenceConstants.PYJAMA_ASSOCIATE_NATURE);
			}
			if (event.getProperty().equals(
					PreferenceConstants.PYJAMA_USE_CUSTOM_COMPILER)) {
				IPreferenceStore prefs = Activator.getDefault()
						.getPreferenceStore();
				fUseCustomCompiler = prefs
						.getBoolean(PreferenceConstants.PYJAMA_USE_CUSTOM_COMPILER);
			}
		}
	};

	/**
	 * The Constructor. Initializes builder with preferences. Adds a property
	 * changed listener for updating when preferences change.
	 */
	public FileBuilder() {
		super();

		IPreferenceStore prefs = Activator.getDefault().getPreferenceStore();

		// set the compiler path
		fCompilerPath = prefs
				.getString(PreferenceConstants.PYJAMA_COMPILER_PATH);

		// set whether we should build
		fCanBuild = prefs
				.getBoolean(PreferenceConstants.PYJAMA_ASSOCIATE_NATURE);

		// set whether we should use a custom compiler
		fUseCustomCompiler = prefs
				.getBoolean(PreferenceConstants.PYJAMA_USE_CUSTOM_COMPILER);

		// add a property change listener when preferences change
		prefs.addPropertyChangeListener(fPropertyChangeListener);
	}

	/**
	 * Destructor equivalent. Current implementation removes the class' property
	 * change listener from the PYJAMA Preference page.
	 */
	public void dispose() {
		// remove the property change listener before class dies
		Activator.getDefault().getPreferenceStore()
				.removePropertyChangeListener(fPropertyChangeListener);
	}

	/**
	 * Builds the resources in a project. Invokes a {@link BuildVisitor} for
	 * full builds, or a ({@link BuildDeltaVisitor} for incremental and auto
	 * builds.
	 * 
	 * @see org.eclipse.core.resources.IncrementalProjectBuilder#build(int,
	 *      java.util.Map, org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	protected IProject[] build(int kind, Map args, IProgressMonitor monitor)
			throws CoreException {

		// check if user wants PYJAMA builds
		if (!fCanBuild)
			return null;

		// determine what type of build was requested
		if (kind == FULL_BUILD) {
			// rebuild all .PYJAMA files in project
			fullBuild(monitor);
		} else {
			IResourceDelta delta = getDelta(getProject());
			if (delta == null) {
				// delta returns null if builder has not been invoked before,
				// or if builder has not been invoked for a very long time
				fullBuild(monitor);
			} else {
				// changes detected, build only changed .PYJAMA resources
				incrementalBuild(delta, monitor);
			}
		}

		new Job("Project Build") {
			protected IStatus run(IProgressMonitor monitor) {

				try {
					// refresh project
					getProject().refreshLocal(IProject.DEPTH_INFINITE, monitor);
					// invoke a full build
					getProject().build(
							IncrementalProjectBuilder.INCREMENTAL_BUILD,
							JavaCore.BUILDER_ID, null, monitor);

					// Getting Markers for test purposes
					IMarker[] markers = getProject().findMarkers(
							IMarker.PROBLEM, true, IProject.DEPTH_INFINITE);
					for (IMarker i : markers) {
						MarkerManager.getInstance().handleJavaMarker(i);
					}
					// TODO: recode this part, refactoring is interesting
					// IMarker[] markerText=
					// getProject().findMarkers(IMarker.TEXT, true,
					// IProject.DEPTH_INFINITE);
					// for (IMarker i : markerText) {
					// MarkerManager.getInstance().handlePJFactoring(i);
					// }
				} catch (CoreException exception) {
					// log.logError(exception);
				}
				return Status.OK_STATUS;
			}
		}.schedule();
		return null;
	}

	/**
	 * Current implementation removes all PYJAMA Problem markers from the
	 * project.
	 * 
	 * @see org.eclipse.core.resources.IncrementalProjectBuilder#clean(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	protected void clean(IProgressMonitor monitor) {
		MarkerManager.getInstance().deletePYJAMAMarkers(getProject());
	}

	private void fullBuild(IProgressMonitor monitor) {
		if (!MarkerManager.getInstance().deletePYJAMAMarkers(getProject()))
			return;

		try {
			// create a BuildVisitor to visit resource and resources children.
			// since this is called on the project, we visit every resource in
			// the workspace
			getProject().accept(new BuildVisitor());

			// getProject().refreshLocal(IProject.DEPTH_INFINITE, monitor);
		} catch (CoreException e) {
		}
	}

	private void incrementalBuild(IResourceDelta delta, IProgressMonitor monitor) {
		try {
			// delta is the root of the tree representing changes in the project
			// we will traverse the tree to evaluate all changes
			delta.accept(new BuildDeltaVisitor());

			// getProject().refreshLocal(IProject.DEPTH_INFINITE, monitor);
		} catch (CoreException e) {
		}
	}

	// --------------------------------------------------------------------------
	// COMPILER RELATED METHODS
	/**
	 * Stream that reads in bytes and stores them. Used when invoking the
	 * default compiler
	 */
	private static class StringOutputStream extends OutputStream {
		private StringBuffer fBuffer;

		public StringOutputStream(StringBuffer buf) {
			super();
			Assert.isNotNull(buf);
			fBuffer = buf;
		}

		@Override
		public void write(int b) throws IOException {
			fBuffer.append((char) b);
		}
	}

	public static void invokeCompiler(final String toCompile,
			final IResource resource) {
		final File projectLocation = resource.getProject().getLocation()
				.toFile();
		// check if User wanted to use their own custom compiler
		if (fUseCustomCompiler) {
			if (fCompilerPath.isEmpty()) {
				return;
			}

			// Dynamically form the run command
			String cmd = getProjectTypeCommand(toCompile, projectLocation);

			Runtime rt = Runtime.getRuntime();
			try {
				// execute command
				Process proc = rt.exec(cmd);

				// establish objects to read output of executed program
				StreamGobbler errorGobbler = new StreamGobbler(
						proc.getErrorStream(), "ERROR");
				StreamGobbler outputGobbler = new StreamGobbler(
						proc.getInputStream(), "OUTPUT");

				errorGobbler.start();
				outputGobbler.start();

				// block until compiler finished
				proc.waitFor();

				errorGobbler.join();
				outputGobbler.join();

				// System.out.println("errorGobbler.getMsg(): " +
				// errorGobbler.getMsg());

				if (!errorGobbler.getMsg().isEmpty()) {
					// error stream was not empty .: some error occured
					// the error message is stored in errorGobbler.getMsg()
					// NOTE: the error message is truncated from what the
					// compiler
					// puts out, because there's a lot of junk that is
					// irrelevant
					String s = errorGobbler.getMsg();
					MarkerInfo m = createMarkerFromErrorString(s);
					MarkerManager.getInstance().reportProblem(resource, m);
				}
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			// User chose to use the default compiler

			// for invoking from the same JVM using a different thread
			// This way requires PTCompiler.jar to be specified on the class
			// path in
			// the manifest file, and means the user does not ever specify the
			// jar file

			Runnable r = new Runnable() {
				@Override
				public void run() {
					// step 1

					// Dynamically form the run command
					String cmd = getProjectTypeCommand(toCompile,
							projectLocation);

					Runtime rt = Runtime.getRuntime();
					try {
						// execute command
						Process proc = rt.exec(cmd);

						// establish objects to read output of executed program
						StreamGobbler errorGobbler = new StreamGobbler(
								proc.getErrorStream(), "ERROR");
						StreamGobbler outputGobbler = new StreamGobbler(
								proc.getInputStream(), "OUTPUT");

						errorGobbler.start();
						outputGobbler.start();

						// block until compiler finished
						proc.waitFor();

						errorGobbler.join();
						outputGobbler.join();

						// System.out.println("errorGobbler.getMsg(): " +
						// errorGobbler.getMsg());

						if (!errorGobbler.getMsg().isEmpty()) {
							// error stream was not empty .: some error occured
							// the error message is stored in
							// errorGobbler.getMsg()
							// NOTE: the error message is truncated from what
							// the compiler
							// puts out, because there's a lot of junk that is
							// irrelevant
							String s = errorGobbler.getMsg();
							MarkerInfo m = createMarkerFromErrorString(s);
							MarkerManager.getInstance().reportProblem(resource,
									m);
						}
					} catch (InterruptedException e) {
						e.printStackTrace();
					} catch (IOException e) {
						e.printStackTrace();
					} finally {
						// step 2 =========================================

						// if not running in debug mode, delete the ptjava files
						IPreferenceStore prefs = Activator.getDefault()
								.getPreferenceStore();
						if (false == prefs
								.getBoolean(PreferenceConstants.PYJAMA_DEBUG_MODE)) {
							if (true == toCompile
									.endsWith(PYJAMA_FILE_EXTENSION)) {
								String toDelete = toCompile.substring(
										0,
										toCompile.length()
												- TODELETE_FILE_EXTENSION
														.length());
								toDelete += TODELETE_FILE_EXTENSION;
								File toDeleteFile = new File(toDelete);
								if (true == toDeleteFile.exists()) {
									toDeleteFile.delete();
								}
							}
						}
					}
				}
			};

			Thread t = new Thread(r);

			StringBuffer sb = new StringBuffer(4096);
			PrintStream printStream = new PrintStream(
					new StringOutputStream(sb));
			try {
				// redirecting error stream while compiler runs
				System.setErr(printStream);
			} catch (SecurityException e) {
				e.printStackTrace();
			}

			t.run();
			try {
				t.join();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			String errString = sb.toString();

			try {
				// return error stream
				System.setErr(System.err);
			} catch (SecurityException e) {
				e.printStackTrace();
			}
			if (!errString.isEmpty() /*
									 * && errString.startsWith(
									 * "********* Failed to parse")
									 */) {
				// parse the error string

				MarkerInfo m = createMarkerFromErrorString(errString);
				MarkerManager.getInstance().reportProblem(resource, m);
			}
		}
	}

	private static MarkerInfo createMarkerFromErrorString(String errorString) {
		Pattern pattern = Pattern
				.compile(".*(jump\\.parser\\.(.*) at (line (\\d+)), column (\\d)+\\.).*");
		Matcher matcher = pattern.matcher(errorString);

		MarkerInfo m = new MarkerInfo();
		if (matcher.find()) {
			// m.fMsg= matcher.group();

			m.fMsg = errorString.substring(0,
					errorString.indexOf("at jump.parser."));

			m.fLocation = matcher.group(1);
			m.fLineNumber = Integer.parseInt(matcher.group(4));
		} else {
			m.fMsg = errorString;
		}
		return m;
	}

	private static MarkerInfo createMarkerFromCompileErrorString(
			String errorString) {
		Pattern pattern = Pattern.compile("####\\[(\\d+)\\]####");
		Matcher matcher = pattern.matcher(errorString);

		MarkerInfo m = new MarkerInfo();
		if (matcher.find()) {
			// m.fMsg= matcher.group();

			m.fMsg = errorString.substring(0,
					errorString.indexOf("at jump.parser."));

			m.fLocation = matcher.group(1);
			m.fLineNumber = Integer.parseInt(matcher.group(4));
		} else {
			m.fMsg = errorString;
		}
		return m;
	}

	// --------------------------------------------------------------------------
	// UTILITY METHODS

	/**
	 * The ID for the builder.
	 */
	public static final String BUILDER_ID = Activator.PLUGIN_ID
			+ ".pyjamaFileBuilder";

	/**
	 * Associates the Pyjama builder with the given project.
	 * 
	 * @param project
	 *            The project to add the builder to
	 */
	public static void addBuilderToProject(IProject project) {
		// cannot modify closed projects
		if (!project.isOpen())
			return;

		// get the description
		IProjectDescription description;
		try {
			description = project.getDescription();
		} catch (CoreException exception) {
			// //PYJAMALog.logError(exception);
			return;
		}

		ICommand[] cmds = description.getBuildSpec();
		for (int j = 0; j < cmds.length; j++) {
			if (cmds[j].getBuilderName().equals(BUILDER_ID))
				return;
		}

		// Associate builder with project.
		ICommand newCmd = description.newCommand();
		newCmd.setBuilderName(BUILDER_ID);
		List<ICommand> newCmds = new ArrayList<ICommand>();
		newCmds.addAll(Arrays.asList(cmds));
		newCmds.add(newCmd);
		description.setBuildSpec((ICommand[]) newCmds
				.toArray(new ICommand[newCmds.size()]));

		// set the project description
		try {
			project.setDescription(description, null);
		} catch (CoreException e) {
			// PYJAMALog.logError(e);
		}
	}

	/**
	 * Removes the builder from the given project. Does nothing if the project
	 * is not open or does not have the builder associated with it.
	 * 
	 * @param project
	 *            The project to remove the builder from
	 */
	public static void removeBuilderFromProject(IProject project) {

		// Cannot modify closed projects.
		if (!project.isOpen())
			return;

		// Get the description.
		IProjectDescription description;
		try {
			description = project.getDescription();
		} catch (CoreException e) {
			// PYJAMALog.logError(e);
			return;
		}

		// Look for builder.
		int index = -1;
		ICommand[] cmds = description.getBuildSpec();
		for (int j = 0; j < cmds.length; j++) {
			if (cmds[j].getBuilderName().equals(BUILDER_ID)) {
				index = j;
				break;
			}
		}
		if (index == -1)
			return;

		// Remove builder from project.
		List<ICommand> newCmds = new ArrayList<ICommand>();
		newCmds.addAll(Arrays.asList(cmds));
		newCmds.remove(index);
		description.setBuildSpec((ICommand[]) newCmds
				.toArray(new ICommand[newCmds.size()]));
		try {
			project.setDescription(description, null);
		} catch (CoreException e) {
			// PYJAMALog.logError(e);
		}
	}

	private static String getProjectTypeCommand(String _toCompile,
			File _projectLocation) {
		String command = null;
		String jarPath = pjplugin.preferences.PreferencePage
				.getDefaultRuntimeJarPath().toString();

		if (jarPath == null || PJConstants.DEFAULT_PATH.equals(jarPath)) {
			if (_projectLocation != null) {
				String libFolderPath = _projectLocation.getAbsolutePath()
						+ File.separator + PJConstants.LIBS_DIRECTORY
						+ File.separator + PJConstants.PJ_JARNAME;
				File pjJarFile = new File(libFolderPath);
				if (pjJarFile.exists()) {
					jarPath = libFolderPath;
					projectType = ProjectType.ANDROID;
				} else {
					libFolderPath = _projectLocation.getAbsolutePath()
							+ File.separator + PJConstants.LIB_DIRECTORY
							+ File.separator + PJConstants.PJ_JARNAME;
					pjJarFile = new File(libFolderPath);
					if (pjJarFile.exists()) {
						jarPath = libFolderPath;
						projectType = ProjectType.DESKTOP;
					}
				}
			}
		}

		if (jarPath != null) {
			command = (JAVA_CMD_OPTION).concat(jarPath).concat(" ")
					.concat(_toCompile);
			if (projectType == ProjectType.ANDROID)
				command = command.concat(" ").concat(ANDROID_CMD_OPTION);
		}

		return command;
	}
}
